<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Event\Form\Enums\GdprType;
use EventPortal\Domain\Event\Form\Enums\QuestionType;
use EventPortal\Domain\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Str;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Enum\Laravel\HasEnums;

class Question extends Model implements AuditableInterface
{
    use HasEnums, Auditable, SoftDeletes;

    /** @var array<string,mixed> */
    protected $enums = [
        'type' => QuestionType::class,
        'gdpr_type' => GdprType::class,
    ];

    /** @var array<string> */
    protected $fillable = [
        'question',
        'type',
        'help_text',
        'help_position',
        'form_step_section_id',
        'gdpr_type',
        'order',
        'uuid',
    ];

    public function stage(): BelongsTo
    {
        return $this->belongsTo(FormStep::class);
    }

    public function options(): HasMany
    {
        return $this->hasMany(QuestionOption::class)->orderBy('order');
    }

    public function validations(): HasMany
    {
        return $this->hasMany(QuestionValidation::class);
    }

    public function scopeForFormStepSection(Builder $query, FormStepSection $formStepSection): Builder
    {
        return $query->where('form_step_section_id', '=', $formStepSection->id);
    }

    public function isGdprSensitive(): bool
    {
        return $this->gdpr_type->isEqual(GdprType::Sensitive());
    }

    public function hasHelpText(): bool
    {
        return $this->help_text !== null;
    }

    public function toArray(): array
    {
        return [
            'id' => $this->id,
            'uuid' => $this->uuid,
            'question' => $this->question,
            'order' => $this->order,
            'help' => $this->hasHelpText(),
            'helpText' => $this->help_text,
            'helpPosition' => $this->help_position,
            'gdprType' => Str::lower($this->gdpr_type),
            'required' => $this->validations()->isRequired()->exists(),
            'component' => [
                'name' => [
                    'label' => $this->type->getLabel(),
                    'value' => $this->type,
                ],
                'options' => $this->options->toArray(),
            ],
        ];
    }
}
