<?php

namespace EventPortal\Domain\Event\Models;

use Carbon\Carbon;
use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Concerns\HasTeams;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Model;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\Table\Concerns\HasTable;
use EventPortal\Domain\Util\Notification\Models\EventNotification;
use EventPortal\Domain\Venue\Models\Venue;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Notifications\Notifiable;
use Illuminate\Support\Arr;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Event extends Model implements AuditableInterface, Statusable
{
    use HasTable,
        AddToFillable,
        AddToCasts,
        Auditable,
        Statable,
        HasStatus,
        HasTeams,
        Notifiable;

    public const KEY = 'event';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'slug',
        'status',
        'email',
        'snippet',
        'description',
    ];

    protected function getGraph(): string
    {
        return 'event';
    }

    public function agendas(): HasMany
    {
        return $this->hasMany(Agenda::class);
    }

    public function delegates(): BelongsToMany
    {
        return $this
            ->belongsToMany(Delegate::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->as('delegates')
            ->withTimestamps();
    }

    public function speakers(): BelongsToMany
    {
        return $this
            ->belongsToMany(Speaker::class)
            ->using(EventSpeaker::class)
            ->withPivot(['id'])
            ->as('speakers')
            ->withTimestamps();
    }

    public function venues(): BelongsToMany
    {
        return $this
            ->belongsToMany(Venue::class)
            ->using(EventVenue::class)
            ->withPivot(['id'])
            ->as('venues')
            ->withTimestamps();
    }

    public function emails(): HasMany
    {
        return $this->hasMany(EventEmail::class);
    }

    public function notifications(): HasMany
    {
        return $this->hasMany(EventNotification::class);
    }

    public function forms(): HasMany
    {
        return $this->hasMany(Form::class);
    }

    public function transitionRouteName(): string
    {
        return 'admin.event.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    public function getStartDateAttribute(): ?Carbon
    {
        return optional($this->agendas()->orderBy('date')->first())->date;
    }

    public function getReadableStartDateAttribute(): ?string
    {
        return optional($this->startDate)->format('dS F Y');
    }

    public function getReadableDatesAttribute(): string
    {
        if ($this->agendas->isEmpty()) {
            return '';
        }

        if ($this->agendas->count() <= 1) {
            return $this->agendas->first()->date->format('d/m/Y');
        }

        return "{$this->agendas->first()->date->format('d/m/Y')} - {$this->agendas->last()->date->format('d/m/Y')}";
    }

    public function getViewableStatusesAttribute(): array
    {
        return collect(
            config("state-machine.{$this->getGraph()}.states")
        )->reject(function (array $state): bool {
            return Arr::get($state, 'metadata.viewable') === false;
        })->map(function (array $state): string {
            return Arr::get($state, 'name');
        })->toArray();
    }

    public function scopeHasDateAfterOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '>=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }

    public function scopeHasDateBeforeOrEqualTo(Builder $query, string $value): Builder
    {
        return $query->whereIn('id', function ($query) use ($value) {
            return $query
                ->select('event_id')
                ->from('agendas')
                ->whereDate('date', '<=', Carbon::createFromFormat('d/m/Y', $value));
        });
    }

    public function scopeViewable(Builder $query): Builder
    {
        return $query->whereIn('status', $this->viewableStatuses);
    }

    public function scopeForDelegate(Builder $query, Delegate $delegate): Builder
    {
        return $query->whereHas('delegates', static function (Builder $query) use ($delegate) {
            return $query->where('delegate_id', $delegate->id);
        });
    }

    public function scopeUpcoming(Builder $query): Builder
    {
        return $query->whereHas('agendas', static function ($query) {
            return $query->whereDate('date', '>', now());
        });
    }

    public function scopePrevious(Builder $query): Builder
    {
        return $query->whereHas('agendas', static function ($query) {
            return $query->whereDate('date', '<', now());
        });
    }
}
